--[[
ObjectInfo

Specialization to show some infos from the object or vehicle in front of player and destroy bales, if the player the bales and area, where the bales are, owned by him

	@author:    Ifko[nator]
	@date:      02.05.2020
    @version:	1.0
]]

ObjectInfo = {};
ObjectInfo.currentModDirectory = g_currentModDirectory;

function ObjectInfo.renderTextObjectInfo(x, y, text, textSize, textOffset, farmColor)
    setTextAlignment(RenderText.ALIGN_CENTER);
    setTextBold(true);
    setTextColor(0, 0, 0, 1);

    renderText(x, y - 0.0018 + textOffset, textSize, text);
        
    setTextColor(unpack(farmColor));
    renderText(x, y + textOffset, textSize, text);

    setTextColor(1, 1, 1, 1);
    setTextAlignment(RenderText.ALIGN_LEFT);
end;

function ObjectInfo.renderFillTypeIcon(icon, iconColor, x, y)
    icon:setColor(unpack(iconColor));
    icon:setPosition(x, y)
    icon:render();
end;

function ObjectInfo.getColorForFarm(farmId, allowFarmColor)
    if farmId ~= nil and allowFarmColor then
        local colorTable = Farm.COLORS[farmId.color];
        
        if colorTable ~= nil then
            return colorTable;
        end;
    end;

    return {1, 1, 1, 1};
end;

function ObjectInfo.getIsTool(xmlFile, TAG)
    local attacherJoints = {
        "frontloader",
        "telehandler",
        "wheelLoader",
        "skidSteer"
    }
   
    for _, attacherJoint in pairs(attacherJoints) do
        if getXMLString(xmlFile, TAG) == attacherJoint then
            return true;
        end;
    end;

    return false;
end;

function ObjectInfo.getWrappingColorName(wrappingColor)
    local wrappingColorString = MathUtil.round(wrappingColor[1], 2) .. " " .. MathUtil.round(wrappingColor[2], 2) .. " " .. MathUtil.round(wrappingColor[3], 2);
    local wrappingColorName = wrappingColorString .. " (r, g, b)";

    if wrappingColorString == "0.99 0.99 0.99" or wrappingColorString == "1 1 1" then
        wrappingColorName = g_i18n:getText("color_white");
    elseif wrappingColorString == "0.05 0.05 0.05" then
        wrappingColorName = g_i18n:getText("color_black");
    elseif wrappingColorString == "0.67 0.38 0.55" then
        wrappingColorName = g_i18n:getText("color_pink");
    elseif wrappingColorString == "0.39 0.52 0.37" then
        wrappingColorName = g_i18n:getText("color_green");
    end;

    return wrappingColorName;
end;

function ObjectInfo:loadMap(name)
    self.cutterHud = Overlay:new(Utils.getFilename("hud/iconCutter.dds", ObjectInfo.currentModDirectory), 0, 0, 0.04, 0.08);

    self.disableButtons = false;
    self.buttonIsActive = false;
    self.showInfo = true;
    self.allowFarmColor = true;
    self.textSize = 1.5;
    self.maxTextSize = 3;
end;

function ObjectInfo:destroyBale()
    if self.objectInRange ~= nil and self.objectInRange:isa(Bale) then
        self:destroyBales(self.objectInRange);

        BaleDestroyEvent:sendEvent(self.objectInRange);
	end;
end;

function ObjectInfo:destroyBales(baleToDestroy)
    if baleToDestroy ~= nil then
        local delta = baleToDestroy:getFillLevel();
        local fillType = baleToDestroy:getFillType();
        local x, y, z = getWorldTranslation(baleToDestroy.meshNode);

        local xzRndm = ((math.random(1, 20)) - 10) / 10;
        local xOffset = math.max(math.min(xzRndm, 0.3), -0.3);
        local zOffset = math.max(math.min(xzRndm, 0.8), -0.1);

        local ex = x + xOffset;
        local ey = y - 0.1;
        local ez = z + zOffset;

        local innerRadius = 0;
        local outerRadius = DensityMapHeightUtil.getDefaultMaxRadius(fillType);
        local levelerNode = 1;

        local dropped, droppedToLine = DensityMapHeightUtil.tipToGroundAroundLine(nil,
            delta,
            fillType,
            x,
            y,
            z,
            ex,
            ey,
            ez,
            innerRadius,
            outerRadius,
            levelerNode,
            false, 
            nil, 
            false
        );

        baleToDestroy:setFillLevel(baleToDestroy:getFillLevel() - dropped);

        if g_currentMission:getIsServer() then
            if baleToDestroy:getFillLevel() <= 0 then
                baleToDestroy:delete();
            end;
        end;
	end;
end;

function ObjectInfo:setShowHideInfo()
    self.showInfo = not self.showInfo;
end;

function ObjectInfo:setTextSize()
    if self.textSize < self.maxTextSize then
        self.textSize = self.textSize + 0.5;
    else
        self.textSize = 1;
    end;
end;

function ObjectInfo:setTextColor()
    self.allowFarmColor = not self.allowFarmColor;
end;

function ObjectInfo:update(dt)
    if g_currentMission:getIsClient() then
        if not self.buttonIsActive then
            _ , self.destroyBaleButton = g_inputBinding:registerActionEvent(InputAction.OBJECT_INFO_DESTROY_BALE, ObjectInfo, ObjectInfo.destroyBale, false, true, false, false);
            _ , self.setShowHideInfoButton = g_inputBinding:registerActionEvent(InputAction.OBJECT_INFO_SET_SHOW_HIDE_INFO, ObjectInfo, ObjectInfo.setShowHideInfo, false, true, false, false);
            _ , self.setTextSizeButton = g_inputBinding:registerActionEvent(InputAction.OBJECT_INFO_SET_TEXT_SIZE, ObjectInfo, ObjectInfo.setTextSize, false, true, false, false);
            _ , self.setTextColorButton = g_inputBinding:registerActionEvent(InputAction.OBJECT_INFO_SET_TEXT_COLOR, ObjectInfo, ObjectInfo.setTextColor, false, true, false, false);

            g_inputBinding:setActionEventTextPriority(self.destroyBaleButton, GS_PRIO_VERY_HIGH);
            g_inputBinding:setActionEventTextPriority(self.setShowHideInfoButton, GS_PRIO_VERY_HIGH);
            g_inputBinding:setActionEventTextPriority(self.setTextSizeButton, GS_PRIO_HIGH);
            g_inputBinding:setActionEventTextPriority(self.setTextColorButton, GS_PRIO_HIGH);

           self.buttonIsActive = true;
        end;

        if g_currentMission.player ~= nil 
	    	and g_currentMission.controlledVehicle == nil  
	    	and g_currentMission.player.baseInformation.currentHandtool == nil 
	    then
            self.objectInRange = nil;

            g_inputBinding:setActionEventText(self.setShowHideInfoButton, g_i18n:getText("helpText_SHOW_INFO"));
            g_inputBinding:setActionEventActive(self.setShowHideInfoButton, true);
            g_inputBinding:setActionEventTextVisibility(self.setShowHideInfoButton, true);

            if self.showInfo then
                g_inputBinding:setActionEventText(self.setShowHideInfoButton, g_i18n:getText("helpText_HIDE_INFO"));
            end;

            g_currentMission.player.showInfo = self.showInfo;

            if g_currentMission.player.lastObjectInRange ~= nil and self.showInfo then
                self.objectInRange = g_currentMission.player.lastObjectInRange;

                if g_seasons ~= nil then
                    if self.objectInRange:isa(Bale) and self.objectInRange.wrappingState == 1 then  
                        self.fermentingProcess = g_currentMission.player.fermentingProcess;
                    end;
                end;

                self.disableButtons = false;
            else
                self.disableButtons = true;
            end;
        else
            self.buttonIsActive = false;
            self.disableButtons = true;

            g_inputBinding:setActionEventActive(self.setShowHideInfoButton, false);

            g_inputBinding:setActionEventTextVisibility(self.setShowHideInfoButton, false);
        end;

        g_inputBinding:setActionEventActive(self.destroyBaleButton, not self.disableButtons);
        g_inputBinding:setActionEventActive(self.setTextSizeButton, not self.disableButtons);
        g_inputBinding:setActionEventActive(self.setTextColorButton, not self.disableButtons);

        g_inputBinding:setActionEventTextVisibility(self.destroyBaleButton, not self.disableButtons);
        g_inputBinding:setActionEventTextVisibility(self.setTextSizeButton, not self.disableButtons);
        g_inputBinding:setActionEventTextVisibility(self.setTextColorButton, not self.disableButtons);
    end;
end;

function ObjectInfo:draw()
    local isEnteredAsPassenger = false;

    if g_universalPassenger ~= nil then
        isEnteredAsPassenger = g_universalPassenger.currentPassengerVehicle ~= nil;
    end;

    if self.buttonIsActive and not isEnteredAsPassenger then
        if self.objectInRange ~= nil then
		    local fillType = g_currentMission.fillTypeManager.fillTypes[self.objectInRange.fillType];
            local fillLevel, capacity, percent, fillTypeName = 0, 0, 0, "";
        
            if self.objectInRange.getFillUnitFillType ~= nil then
                fillType = g_currentMission.fillTypeManager.fillTypes[self.objectInRange:getFillUnitFillType(1)];
            end;

            if self.objectInRange.fillLevel ~= nil then
                fillLevel = self.objectInRange.fillLevel;
            elseif self.objectInRange.getFillUnitFillLevel ~= nil then
                fillLevel = self.objectInRange:getFillUnitFillLevel(1);
                capacity = self.objectInRange:getFillUnitCapacity(1);
            end;

            local activeFarm = g_farmManager:getFarmById(g_currentMission:getFarmId());
            local objectFarm = g_farmManager:getFarmById(self.objectInRange:getOwnerFarmId());

            local textObjectInfos = "";
            local owner = g_i18n:getText("objectInfo_owner") .. ": " .. objectFarm.name;
            local allowDestroyBale = true;
            
            if g_gui.currentGui == nil then
                local x, y, z = 0, 0, 0;
    
                if self.objectInRange.meshNode ~= nil then
                    if entityExists(self.objectInRange.nodeId) then    
                        x, y, z = getWorldTranslation(self.objectInRange.meshNode);
                    end;
                else
                    if self.objectInRange.components ~= nil and entityExists(self.objectInRange.components[1].node) then     
                        x, y, z = getWorldTranslation(self.objectInRange.components[1].node);
                    
                        y = y + 0.45;

                        local isWeight = getXMLString(self.objectInRange.xmlFile, "vehicle.storeData.functions.function(0)") == "$l10n_function_weight";
                        
                        if self.objectInRange.spec_wheels ~= nil 
                            and not isWeight 
                            and not ObjectInfo.getIsTool(self.objectInRange.xmlFile, "vehicle.attachable.inputAttacherJoints.inputAttacherJoint(0)#jointType")
                            and not string.find(self.objectInRange.xmlFile, "barrels")
                        then
                            y = y + 0.75;
                        end;
                    end;
                end;
                    
                local sx, sy, sz = project(x, y, z);
                local objectIsInRange = sx > -1 and sx < 2 and sy > -1 and sy < 2 and sz <= 1;

                if fillType ~= nil then
                    if not self.objectInRange:isa(Bale) and fillType.name ~= "UNKNOWN" then
                        percent = fillLevel / capacity * 100;
                    end;

                    local wrappingState = Utils.getNoNil(self.objectInRange.wrappingState, 0); 
                    local wrappingColor = self.objectInRange.wrappingColor;
                    local baseValue = 0;

                    if wrappingState > 0 and wrappingState < 1 then
                        self.objectInRange:setWrappingState(1);
                    end;
                
                    if self.objectInRange.getValue ~= nil then
                        baseValue = self.objectInRange:getValue();
                    end;

                    local realWeight = 0;
                
                    fillTypeName = fillType.title or fillType.name;
                    realWeight = (fillLevel * fillType.massPerLiter) * 1000;
                    allowDestroyBale = g_densityMapHeightManager:getDensityMapHeightTypeByFillTypeIndex(g_currentMission.fillTypeManager:getFillTypeIndexByName(fillType.name)) ~= nil;

                    if self.objectInRange.meshNode ~= nil then
                        local playerX, _, playerZ = localToLocal(g_currentMission.player.cameraNode, g_currentMission.player.lightNode, 0, 0, 0);
                        local lightX, _, lightZ = localToWorld(g_currentMission.player.lightNode, playerX, 0, playerZ);

                        if not g_currentMission.accessHandler:canFarmAccessLand(activeFarm.farmId, lightX, lightZ) then
                            allowDestroyBale = false;
                        end;

                        if objectFarm.farmId == 0 or objectFarm ~= activeFarm then
                            allowDestroyBale = false;
                        end;
                    end;

                    g_inputBinding:setActionEventActive(self.destroyBaleButton, self.objectInRange:isa(Bale) and allowDestroyBale);
                    g_inputBinding:setActionEventTextVisibility(self.destroyBaleButton, self.objectInRange:isa(Bale) and allowDestroyBale);

                    local moneySmybol = g_i18n:getText("unit_euroShort");
                
		            if g_gameSettings ~= nil then
		            	if g_gameSettings:getValue("moneyUnit") == 2 then
		            		moneySmybol = g_i18n:getText("unit_dollarShort");
		            	elseif g_gameSettings:getValue("moneyUnit") == 3 then
		            		moneySmybol = g_i18n:getText("unit_poundShort");
		            	end;
                    end;

                    if self.objectInRange.spec_fillUnit ~= nil then
                        local availableFillUnits = #self.objectInRange.spec_fillUnit.fillUnits;
                        local emptyFillUnits = 0;

                        for fillUnitNumber, fillUnit in pairs(self.objectInRange.spec_fillUnit.fillUnits) do
                            local fillType = nil;

                            if fillUnit.fillType == FillType.UNKNOWN or fillUnit.fillLevel == 0 or fillUnit.capacity == math.huge then
                                availableFillUnits = availableFillUnits - 1;
                                emptyFillUnits = emptyFillUnits + 1;
                            else
                                fillType = g_currentMission.fillTypeManager.fillTypes[self.objectInRange:getFillUnitFillType(fillUnitNumber)];
                            end;

                            if fillType ~= nil then
                                local fillUnitNumberText = "";
                                local title = fillType.title;

                                if fillType.hudOverlayFilename ~= "" and not string.find(fillType.hudOverlayFilename, "FS19_MoreBunkersilo") then --## more bunker silo mod not supportet due wrong registered fill types!
                                    local overlay = Overlay:new(fillType.hudOverlayFilename, 0, 0, 0.04, 0.065);

                                    if availableFillUnits == 1 then    
                                        ObjectInfo.renderFillTypeIcon(overlay, {1, 1, 1, 0.5}, sx - 0.02, sy + 0.02);
                                    else
                                        local hundredth = availableFillUnits / 100;
                                        local diff = 0.01 * availableFillUnits;
                                        local startX = sx - hundredth - (0.04 + diff);

                                        ObjectInfo.renderFillTypeIcon(overlay, {1, 1, 1, 0.5}, startX + (0.04 * (fillUnitNumber - emptyFillUnits)), sy + 0.02);
                                    end;
                                end;

                                if availableFillUnits > 1 then
                                    fillUnitNumberText = " (" .. fillUnitNumber - emptyFillUnits .. ")";
                                end;

                                if fillType.name == "DEF" then
                                    title = "Ad-Blue";
                                end;

                                if not self.objectInRange:isa(Bale) and fillType.name ~= "UNKNOWN" then
                                    percent = math.floor(fillUnit.fillLevel / fillUnit.capacity * 100);
                                end;

                                if fillUnitNumber - emptyFillUnits == 1 then
                                    if fillUnit.fillLevel > 0 then
                                        textObjectInfos = g_i18n:getText("objectInfo_fillType") .. fillUnitNumberText .. ": " .. title .. " " .. string.format("%0.f l", fillUnit.fillLevel) .. " (" .. percent .. "%)";
                                    else
                                        textObjectInfos = g_i18n:getText("objectInfo_fillType") .. fillUnitNumberText .. ": " .. string.format("%0.f l", fillUnit.fillLevel) .. " (" .. percent .. "%)";
                                    end;
                                else
                                    if fillUnit.fillLevel > 0 then
                                        textObjectInfos = textObjectInfos .. "\n" .. g_i18n:getText("objectInfo_fillType") .. fillUnitNumberText .. ": " .. title .. " " .. string.format("%0.f l", fillUnit.fillLevel) .. " (" .. percent .. "%)";
                                    else
                                        textObjectInfos = textObjectInfos .. "\n" .. g_i18n:getText("objectInfo_fillType") .. fillUnitNumberText .. ": " .. string.format("%0.f l", fillUnit.fillLevel) .. " (" .. percent .. "%)";
                                    end;
                                end;
                            end;
                        end;
                    else
                        if not self.objectInRange:isa(Bale) then    
                            textObjectInfos = g_i18n:getText("objectInfo_percent") .. ": " .. string.format("%0.f l", fillLevel) .. " (" .. math.floor(percent) .. "%)";
                        else
                            textObjectInfos = g_i18n:getText("objectInfo_percent") .. ": " .. string.format("%0.f l", fillLevel);
                        end;
                    end;

                    if fillLevel > 0 and self.objectInRange.spec_fillUnit == nil then
                        textObjectInfos = g_i18n:getText("objectInfo_fillType") .. ": " .. fillTypeName .. "\n" .. textObjectInfos;

                        if baseValue > 0 then
                            textObjectInfos = textObjectInfos .. "\n " .. g_i18n:getText("objectInfo_weight") .. ": " .. string.format("%0.f kg", realWeight) .. "\n " .. g_i18n:getText("objectInfo_price") .. ": " .. g_i18n:formatNumber(baseValue, 0) .. moneySmybol;
                        end;
                    
                        if wrappingState > 0 then    
                            textObjectInfos = textObjectInfos .. "\n" .. g_i18n:getText("objectInfo_wrappingState") .. ": " .. tostring(wrappingState) .. "\n" ..  g_i18n:getText("objectInfo_wrappingColor") .. ": " .. ObjectInfo.getWrappingColorName(self.objectInRange.wrappingColor);
                        end;

                        if g_seasons ~= nil then
                            if self.fermentingProcess ~= nil then
                                if fillType.name == "GRASS_WINDROW" then
                                    textObjectInfos = textObjectInfos .. "\n" .. g_i18n:getText("objectInfo_fermentingProcess") .. ": " .. MathUtil.round(self.fermentingProcess * 100, 1) .. "%";
                                end;
                            end;
                        end;
                    end;
                end;

                if self.objectInRange.spec_wearable ~= nil then
                    local currentDamage = g_i18n:getText("objectInfo_currentDamage") .. ": " .. MathUtil.round(self.objectInRange:getWearTotalAmount() * 100, 1) .. "%";
    
                    if textObjectInfos ~= "" then
                        textObjectInfos = textObjectInfos .. "\n" .. currentDamage;
                    else
                        textObjectInfos = currentDamage;
                    end;
                end;
    
                if textObjectInfos ~= "" then
                    textObjectInfos = textObjectInfos .. "\n" .. owner;
                else
                    textObjectInfos = owner;
                end;

                g_inputBinding:setActionEventText(self.setTextColorButton, g_i18n:getText("helpText_TEXT_COLOR_TO_FARM_COLOR"));

                if self.allowFarmColor then
                    g_inputBinding:setActionEventText(self.setTextColorButton, g_i18n:getText("helpText_TEXT_COLOR_TO_WHITE"));
                end;

                if objectIsInRange then
                    if self.objectInRange:isa(Bale) then
                        if allowDestroyBale then
                            ObjectInfo.renderFillTypeIcon(self.cutterHud, {1, 1, 1, 0.5}, sx - 0.02, sy + 0.02);
                        else
                            if fillType ~= nil then
                                if fillType.hudOverlayFilename ~= "" then
                                    local overlay = Overlay:new(Utils.getFilename(fillType.hudOverlayFilename, self.baseDirectory), 0, 0, 0.04, 0.065);
                                    
                                    ObjectInfo.renderFillTypeIcon(overlay, {1, 1, 1, 0.5}, sx - 0.02, sy + 0.02);
                                end;
                            end;
                        end;
                    end;
                    
                    local farmColor = ObjectInfo.getColorForFarm(objectFarm, self.allowFarmColor);
                    local textSize = 0.01 * self.textSize;
            
                    ObjectInfo.renderTextObjectInfo(sx, sy, textObjectInfos, getCorrectTextSize(textSize), 0, farmColor);
                end;
            end;
        end;
    end;
end;

function ObjectInfo:updateTick(dt)end;
function ObjectInfo:deleteMap()end;
function ObjectInfo:keyEvent(unicode, sym, modifier, isDown)end;
function ObjectInfo:mouseEvent(posX, posY, isDown, isUp, button)end;

addModEventListener(ObjectInfo);

BaleDestroyEvent = {};
BaleDestroyEvent_mt = Class(BaleDestroyEvent, Event);

InitEventClass(BaleDestroyEvent, "BaleDestroyEvent");

function BaleDestroyEvent:emptyNew()
    local self = Event:new(BaleDestroyEvent_mt);
	
    return self;
end;

function BaleDestroyEvent:new(baleToDestroy)
    local self = BaleDestroyEvent:emptyNew();

    self.baleToDestroy = NetworkUtil.getObjectId(baleToDestroy);

    return self;
end;

function BaleDestroyEvent:writeStream(streamId, connection)
    NetworkUtil.writeNodeObjectId(streamId, self.baleToDestroy);
end;

function BaleDestroyEvent:readStream(streamId, connection)
    self.baleToDestroy = NetworkUtil.readNodeObjectId(streamId);

    self:run(connection);
end;

function BaleDestroyEvent:run(connection)
    local baleToDestroy = NetworkUtil.getObject(self.baleToDestroy);

    if not connection:getIsServer() then
        g_server:broadcastEvent(BaleDestroyEvent:new(baleToDestroy), false, connection);
    end;

    ObjectInfo:destroyBales(baleToDestroy);
end;

function BaleDestroyEvent:sendEvent(baleToDestroy)
    local destroyEvent = BaleDestroyEvent:new(baleToDestroy);

    if g_currentMission:getIsServer() then
        g_server:broadcastEvent(destroyEvent, false);
    else
        g_client:getServerConnection():sendEvent(destroyEvent);
    end;
end;
