--[[
PlayerOverwrites

Specialization to overwrite some player functions

	@author:    Ifko[nator]
	@date:      02.05.2020
	@version:	1.0
]]

PlayerOverwrites = {};

function PlayerOverwrites:load()
    self.showInfo = true;
    self.fermentingProcess = 0;
    self.foundVehicleOrObject = nil;
    self.isVehicleOrObjectInRange = false;
end;

function PlayerOverwrites:update(dt)
   if self.isServer and (self.isEntered or self.isControlled) then
        self.lastObjectInRange = nil;
        
        if self.isVehicleOrObjectInRange then
			local object = g_currentMission:getNodeObject(self.foundVehicleOrObject);
            
            if object ~= nil then
                self.lastObjectInRange = object;
            end;
        end;
    end;
end;

function PlayerOverwrites:writeStream(streamId, connection, dirtyMask)
    streamWriteFloat32(streamId, self.fermentingProcess);
end;

function PlayerOverwrites:readStream(streamId, timestamp, connection)
    self.fermentingProcess = streamReadFloat32(streamId);
end;

function PlayerOverwrites:writeUpdateStream(streamId, connection, dirtyMask)
    if not connection:getIsServer() then
        NetworkUtil.writeNodeObject(streamId, self.lastObjectInRange);

        streamWriteFloat32(streamId, self.fermentingProcess);
    end;
end;

function PlayerOverwrites:readUpdateStream(streamId, timestamp, connection)
    if connection:getIsServer() then
        self.lastObjectInRange = NetworkUtil.readNodeObject(streamId);

        self.fermentingProcess = streamReadFloat32(streamId);
    end;
end;

function PlayerOverwrites:writeStreamBale(streamId, connection, dirtyMask)
    streamWriteFloat32(streamId, self.ownerFarmId);
end;

function PlayerOverwrites:readStreamBale(streamId, timestamp, connection)
    self.ownerFarmId = streamReadFloat32(streamId);
end;

function PlayerOverwrites:pickUpObjectRaycastCallback(superFunc, hitObjectId, x, y, z, distance)
    if hitObjectId ~= g_currentMission.terrainDetailId and Player.PICKED_UP_OBJECTS[hitObjectId] ~= true then
        if getRigidBodyType(hitObjectId) == "Dynamic" then

            local mass = getMass(hitObjectId);
            
            --## check if mounted:
            local canBePickedUp = true;
            local object = g_currentMission:getNodeObject(hitObjectId);

            if object ~= nil then
                if object.dynamicMountObject ~= nil then
                    canBePickedUp = false
                end;

                if object.getTotalMass ~= nil then
                    mass = object:getTotalMass()
                end;

                if object.getCanBePickedUp ~= nil then
                    if not object:getCanBePickedUp(self) then
                        canBePickedUp = false;
                    end;
                end;
            end;

            self.foundVehicleOrObject = hitObjectId;

            if canBePickedUp then
                self.lastFoundObject = hitObjectId;
                self.lastFoundObjectMass = mass;
                self.lastFoundObjectHitPoint = {x, y, z};
            end;

            --## only consider first potentially valid object
            return not canBePickedUp; --## fix
        end;
    end;

    return true;
end;

function PlayerOverwrites:checkObjectInRange(superFunc)
	if self.isControlled and self.isServer then
		if not self.isCarryingObject then
			local x, y, z = localToWorld(self.cameraNode, 0, 0, 1);
            local dx, dy, dz = localDirectionToWorld(self.cameraNode, 0, 0, -1);
            
			self.lastFoundObject = nil;
            self.lastFoundObjectHitPoint = nil;
            self.foundVehicleOrObject = nil;

			raycastAll(x, y, z, dx, dy, dz, "pickUpObjectRaycastCallback", Player.MAX_PICKABLE_OBJECT_DISTANCE, self);

            self.isObjectInRange = self.lastFoundObject ~= nil;
            self.isVehicleOrObjectInRange = self.foundVehicleOrObject ~= nil;
		elseif self.pickedUpObject ~= nil and not entityExists(self.pickedUpObject) then
            Player.PICKED_UP_OBJECTS[self.pickedUpObject] = false;
            
			self.pickedUpObject = nil;
			self.pickedUpObjectJointId = nil;
			self.isCarryingObject = false;
		end;
	end;
end;

local manureSystemName = "";

for _, mod in pairs(g_modManager.mods) do
    if _G[tostring(mod.modName)].HosePlayer ~= nil then		
        if g_modIsLoaded[tostring(mod.modName)] then	
            manureSystemName = _G[tostring(mod.modName)];
            break;
        end;
    end;
end;

if manureSystemName ~= "" then
    manureSystemName.HosePlayer.inj_player_pickUpObjectRaycastCallback = PlayerOverwrites.pickUpObjectRaycastCallback;
else
    Player.pickUpObjectRaycastCallback = Utils.overwrittenFunction(Player.pickUpObjectRaycastCallback, PlayerOverwrites.pickUpObjectRaycastCallback);
end;

Player.load = Utils.appendedFunction(Player.load, PlayerOverwrites.load);
Player.update = Utils.appendedFunction(Player.update, PlayerOverwrites.update);
Player.writeStream = Utils.appendedFunction(Player.writeStream, PlayerOverwrites.writeStream);
Player.writeUpdateStream = Utils.appendedFunction(Player.writeUpdateStream, PlayerOverwrites.writeUpdateStream);
Player.readStream = Utils.appendedFunction(Player.readStream, PlayerOverwrites.readStream);
Player.readUpdateStream = Utils.appendedFunction(Player.readUpdateStream, PlayerOverwrites.readUpdateStream);
Player.checkObjectInRange = Utils.overwrittenFunction(Player.checkObjectInRange, PlayerOverwrites.checkObjectInRange);

Bale.writeStream = Utils.appendedFunction(Bale.writeStream, PlayerOverwrites.writeStreamBale);
Bale.readStream = Utils.appendedFunction(Bale.readStream, PlayerOverwrites.readStreamBale);